<?php
/**
 * @author Amasty Team
 * @copyright Copyright (c) 2019 Amasty (https://www.amasty.com)
 * @package Amasty_Coupons
 */


namespace Amasty\Coupons\Plugin;

use Magento\SalesRule\Model\Utility;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\App\State;
use Amasty\Coupons\Helper\Data;
use Amasty\Coupons\Model\CouponRenderer;
use Magento\SalesRule\Model\Rule;
use Magento\Quote\Model\Quote\Address;
use Magento\Backend\App\Area\FrontNameResolver;
use Magento\Framework\Exception\LocalizedException;

/**
 * Class PluginUtility
 */
class PluginUtility
{
    /**
     * @var RequestInterface
     */
    private $request;

    /**
     * @var State
     */
    private $appState;

    /**
     * @var Data
     */
    private $helper;

    /**
     * @var CouponRenderer
     */
    private $couponRenderer;

    public function __construct(
        RequestInterface $request,
        State $appState,
        Data $helper,
        CouponRenderer $couponRenderer
    ) {
        $this->request = $request;
        $this->appState = $appState;
        $this->helper = $helper;
        $this->couponRenderer = $couponRenderer;
    }

    /**
     * Check if rule can be applied for specific address/quote/customer
     * @param Utility $subject
     * @param \Closure $proceed
     * @param Rule $rule
     * @param Address $address
     *
     * @throws LocalizedException
     * @return bool
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     */
    public function aroundCanProcessRule(Utility $subject, \Closure $proceed, $rule, $address)
    {
        $coupons = $this->helper->getRealAppliedCodes(true, $address);
        $userKey = $this->couponRenderer->findCouponInArray($rule->getCode(), $coupons);
        if ($userKey !== false) {
            $originalCouponCode = $address->getQuote()->getCouponCode();
            if ($this->couponRenderer->isCouponUnique($rule->getCode())) {
                $address->getQuote()->setCouponCode($coupons[$userKey]);
            } else {
                $address->getQuote()->setCouponCode(implode(",", $coupons));
            }

            if ($proceed($rule, $address)) {
                //restore original coupon
                if ($this->appState->getAreaCode() == FrontNameResolver::AREA_CODE) {
                    $postParams = $this->request->getPost()->toArray();
                    $postParams['order']['coupon']['code'] = $address->getQuote()->getCouponCode();
                    $this->request->setPost(new \Zend\Stdlib\Parameters($postParams));
                }

                return true;
            } else {
                //restore original coupon
                $address->getQuote()->setCouponCode($originalCouponCode);
                return false;
            }
        }
        return $proceed($rule, $address);
    }
}
